package laskuri3;

import java.awt.*;
import javax.swing.JLabel;
import javax.swing.SwingConstants;

/**
 * Title:        Autolaskuri for Java with Beans
 * Description:  Autolaskuri for Java using beans
 * Copyright:    Copyright (c) 2001
 * Company:      jyu
 * @author Vesa Lappalainen & Miika Nurminen
 * @version 1.1 - 2004/08/28
 *
 * Laskurikomponentti. OnChange-tapahtuman mrittelyss otettu mallia
 * VesterHolm & Kypp: Java-ohjelmointi, luku 18. JavaBeans.
 * Mrittely on tehty ksin. JBuilderilla voitaisiin mys kytt
 * BeansExpressia (Bean-vlilehti)
 */

import java.util.Vector;

public class BeanLaskuri extends JLabel {
  BorderLayout borderLayout1 = new BorderLayout();

  /**
   * Tapahtumankuuntelijat siltn vektoriin.
   * Vektorin sislt ei sarjallisteta.
   */
  private transient Vector changeListeners;

  public BeanLaskuri() {
    try {
      jbInit();
    }
    catch(Exception ex) {
      ex.printStackTrace();
    }
  }
  private void jbInit() throws Exception {
    this.setBackground(Color.cyan);
    this.setFont(new java.awt.Font("Dialog", 1, 18));
    this.setOpaque(true);
    this.setPreferredSize(new Dimension(100, 50));
    this.setHorizontalAlignment(SwingConstants.CENTER);
    this.setHorizontalTextPosition(SwingConstants.CENTER);
    this.setText("0");
    this.setLayout(borderLayout1);
  }
  public int inc(int i) {
    int n = getCount() + 1;
    setCount(n);
    return n;
  }

  public void setCount(int count) {
    int diff = getCount()-count;
    setText(""+count);
    this.fireOnChange(new BeanLaskuriEvent(this,diff));
  }
  public int getCount() {
    return Integer.parseInt(getText());
  }

  /**
   *  Tapahtumankuuntelijan rekisterinnin purku
   *  (poistaa kuuntelijan vektorista)
   */
  public synchronized void removeBeanLaskuriListener(BeanLaskuriListener l) {
    if (changeListeners != null && changeListeners.contains(l)) {
      Vector v = (Vector) changeListeners.clone();
      v.removeElement(l);
      changeListeners = v;
    }
  }

  /**
   * Tapahtumankuuntelijan rekisterinti
   * (lis kuuntelijan vektoriin)
   * */
  public synchronized void addBeanLaskuriListener(BeanLaskuriListener l) {
    Vector v = changeListeners == null ?
            new Vector(2) : (Vector) changeListeners.clone();
    if (!v.contains(l)) {
      v.addElement(l);
      changeListeners = v;
    }
  }

  /**
   * Tapahtuman aiheuttamiseen kytettv metodi.
   * Metodi kutsuu jokaisen kuuntelijan onChange-metodia.
   */
  protected void fireOnChange(BeanLaskuriEvent e) {
    Vector listeners;
    synchronized(this) {
      listeners = changeListeners;
    }
    if (listeners != null) {
      int count = listeners.size();
      for (int i = 0; i < count; i++) {
        ((BeanLaskuriListener)listeners.elementAt(i)).onChange(e);
      }
    }
  }

}